/* 
* Copyright (C) 2024 Dominik Meyer
*  
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*  
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*  
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <Redmine-CLI/Command/Issue.hpp>
#include <Redmine-CLI/Redmine.hpp>
#include "Redmine/API.hpp"
#include "Redmine/Filter.hpp"
#include "nlohmann/json_fwd.hpp"
#include <tableprinter/tableprinter.hpp>

Command::Issue::Issue(RedmineCLI::Redmine *r)
:
redmine_(r),
output_(TEXT),
projectId_(-1),
userId_(-1)
{

    CLI::App* issue = redmine_->getCLI().add_subcommand("issue", 
                                       "Api Calls on issues accessible by token owner")->require_subcommand();
    
    
    list_ = issue->add_subcommand("list", "List issues accessible by the token owner");
    list_->needs(redmine_->getTokenOption())->needs(redmine_->getUrlOption());
    list_->callback([&](){getList_();});
    list_->add_option("-o,--output", output_, "The Output Format to use. Supported: TEXT, YAML")->transform(CLI::CheckedTransformer(outputMap, CLI::ignore_case));    
    list_->add_option("-p,--project",projectId_,"Filter issues by project id");
    list_->add_option("--userid", userId_,"Filter issues by user id");
}   

void Command::Issue::printIssueListText_(const std::vector<Redmine::Issue> &issues) const
{
    tableprinter::printer p
    {
        {
            { tableprinter::name { "ID" }      , tableprinter::width { 8 }  } ,
            { tableprinter::name { "Subject" }    , tableprinter::width { 80 } } ,
            { tableprinter::name { "Project" } , tableprinter::width { 25 } } ,
            {tableprinter::name { "Status" }, tableprinter::width { 15 }}
        } ,
        { std::cout }
    };

    p.sanity_check();
    p.print_headers();

    for (auto it = issues.begin(); it!= issues.end(); ++it)
    {
        p.print(it->getId(), it->getSubject(), it->getProject(), it->getStatus());       
    }

}

void Command::Issue::printIssueListJson_(const std::vector<Redmine::Issue> &issues) const
{
    nlohmann::json data = nlohmann::json::array();
    
    for (auto it = issues.begin(); it!= issues.end(); ++it)
    {
        nlohmann::json issue;
        issue["id"] = it->getId();
        issue["subject"] = it->getSubject();
        issue["project"] = it->getProject();
        issue["project_id"] = it->getProjectId();
        issue["status"] = it->getStatus();
        issue["status_id"] = it->getStatusID();

        data.push_back(issue);
    }
    
    std::cout << data.dump(2);
}

void Command::Issue::getList_() const
{
    Redmine::API client{redmine_->getUrl(), redmine_->getToken()};

    Redmine::Filter filter{};

    if (projectId_ > -1)
    {
        filter.filterByProjectID(projectId_);
    }
    
    if (userId_ > -1)
    {
        filter.filterByAssignedUserId(userId_);
    }

    auto issues = client.getIssues(filter);

    if (output_ == TEXT)
    {
        printIssueListText_(issues);
    }
    else if (output_ == JSON)
    {
        printIssueListJson_(issues);
    }

    return;
}