/* 
* Copyright (C) 2024 Dominik Meyer
*  
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*  
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*  
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Redmine/IssueStatus.hpp"
#include <Redmine-CLI/Command/IssueStatus.hpp>
#include <Redmine-CLI/Redmine.hpp>
#include <vector>
#include "Redmine/API.hpp"
#include "Redmine/User.hpp"
#include "nlohmann/json_fwd.hpp"
#include "tableprinter/tableprinter.hpp"
#define LOGURU_WITH_STREAMS 1
#include <loguru.hpp>

Command::IssueStatus::IssueStatus(RedmineCLI::Redmine *r)
:
redmine_(r),
output_(TEXT)
{

    CLI::App* issuestatus = redmine_->getCLI().add_subcommand("issuestatus", 
                                       "Commands for working with Issue Stati")
                                       ->require_subcommand();


    list_ = issuestatus->add_subcommand("list", "Get the list of possible issue stati");
    list_->callback([&](){getList_();});
    list_->needs(redmine_->getTokenOption())->needs(redmine_->getUrlOption());
    list_->add_option("-o,--output", output_, "The Output Format to use. Supported: TEXT, JSON")->transform(CLI::CheckedTransformer(outputMap, CLI::ignore_case)); 

}
void Command::IssueStatus::printListJson_(const std::vector<Redmine::IssueStatus> &list) const
{
    nlohmann::json data;
    for (const auto& status : list)
    {
        nlohmann::json stat;
        stat["id"] = status.getId();
        stat["name"] = status.getName();
        stat["closesIssue"] = status.isClosed();
        data.push_back(stat);
    }
    std::cout << data.dump(2) << std::endl;
}

void Command::IssueStatus::printListText_(const std::vector<Redmine::IssueStatus> &list) const
{
    tableprinter::printer p
    {
        {
            { tableprinter::name { "ID" }      , tableprinter::width { 8 }  } ,
            { tableprinter::name { "Name" }    , tableprinter::width { 20 } } ,
            { tableprinter::name { "closesIssue" } , tableprinter::width { 14 } }
        } ,
        { std::cout }
    };

    p.sanity_check();
    p.print_headers();

    for (auto status : list)
    {
        p.print(status.getId(), status.getName(), status.isClosed() ? "true" : "false");   
    }

}

void Command::IssueStatus::getList_() const
{

    Redmine::API client{redmine_->getUrl(), redmine_->getToken()};

    std::vector<Redmine::IssueStatus> list = client.getIssueStatusList();

    if (output_ == JSON)
    {
        printListJson_(list);
    }
    else if (output_ == TEXT)
    {
        printListText_(list);
    }

}
