/* 
* Copyright (C) 2024 Dominik Meyer
*  
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*  
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*  
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <Redmine-CLI/Command/Project.hpp>
#include <Redmine-CLI/Redmine.hpp>
#include "Redmine/API.hpp"
#include "nlohmann/json_fwd.hpp"
#include <string>
#include <tableprinter/tableprinter.hpp>

Command::Project::Project(RedmineCLI::Redmine *r)
:
redmine_(r),
output_(TEXT),
limit_(100)
{

    CLI::App* project = redmine_->getCLI().add_subcommand("project", 
                                       "Api Calls on projects accessible by token owner");
    
    
    list_ = project->add_subcommand("list", "List all projects accessible by the token owner");
    list_->needs(redmine_->getTokenOption())->needs(redmine_->getUrlOption());
    list_->callback([&](){getList_();});
    list_->add_option("-f,--fields", fields_, "The fields of the project object to print. Supported: name, id");
    list_->add_option("-o,--output", output_, "The Output Format to use. Supported: TEXT, YAML")->transform(CLI::CheckedTransformer(outputMap, CLI::ignore_case));    
    list_->add_option("-l,--limit",limit_,"The maximum number of projects to return. (Default: " + std::to_string(limit_) +" )");


    upload_ = project->add_subcommand("upload", "Upload a File to the Project");
    upload_->needs(redmine_->getTokenOption())->needs(redmine_->getUrlOption());
    upload_->callback([&](){uploadFile_();});
    upload_->add_option("-p,--project", projectId_, "The ID of the Project to upload the file to")->required()->check(CLI::NonNegativeNumber);
    upload_->add_option("-f,--filename", fileName_, "The name of the file to upload")->required();
    upload_->add_option("-d,--description", fileDescription_, "The Description of the file")->required();
    upload_->add_option("-v,--version", fileVersion_,"The ID of the Version the file is related to. Use non existing version id for no version.")->required()->check(CLI::NonNegativeNumber);
    upload_->add_option("file", filePath_, "The path to the file to upload")->required()->check(CLI::ExistingFile);

}   

void Command::Project::uploadFile_() const
{
    Redmine::API client{redmine_->getUrl(), redmine_->getToken()};
    
    client.uploadFileToProject(projectId_, filePath_, fileName_, fileDescription_,fileVersion_);
}

void Command::Project::printProjectList_(const std::vector<Redmine::Project> &projects) const
{
    bool hasName   = std::find(fields_.begin(), fields_.end(),"name") != fields_.end() || fields_.empty();

    bool hasId = std::find(fields_.begin(), fields_.end(),"id") != fields_.end() || fields_.empty();

    bool hasStrI = std::find(fields_.begin(), fields_.end(),"strid") != fields_.end() || fields_.empty();

    if (output_ == TEXT)
    {
        printProjectListText_(projects, hasName, hasId, hasStrI);
    }
    else if (output_ == JSON)
    {
        printProjectListJson_(projects, hasName, hasId, hasStrI);
    }

}

void Command::Project::printProjectListText_(const std::vector<Redmine::Project> &projects, bool printName, bool printID, bool printIdentifier) const
{
    tableprinter::printer p
    {
        {
            { tableprinter::name { "ID" }      , tableprinter::width { 8 }  } ,
            { tableprinter::name { "Identifier" }    , tableprinter::width { 30 } } ,
            { tableprinter::name { "Name" } , tableprinter::width { 40 } } ,
        } ,
        { std::cout }
    };

    p.sanity_check();
    p.print_headers();

    for (auto it = projects.begin(); it!= projects.end(); ++it)
    {
        if (printID && printIdentifier && printName)
        {
            p.print(it->getId(), it->getStringID(), it->getName());       
        }
        else if (printID && printIdentifier && !printName)
        {
            p.print(it->getId(), it->getStringID());
        }
        else if (printID && !printIdentifier && printName)
        {
            p.print(it->getId(), it->getName());
        }
        else if (!printID && printIdentifier && printName)
        {
            p.print(it->getStringID(), it->getName());
        }
    }

}

void Command::Project::printProjectListJson_(const std::vector<Redmine::Project> &projects, bool printName, bool printID, bool printIdentifier) const
{
    nlohmann::json data = nlohmann::json::array();
    
    for (auto it = projects.begin(); it!= projects.end(); ++it)
    {
        nlohmann::json project;

        if (printID)
        {
            project["id"]=it->getId();
        }

        if (printIdentifier)
        {
            project["identifier"]=it->getStringID();
        }

        if (printName)
        {
            project["name"]=it->getName();
        }

        data.push_back(project);
    }
    
    std::cout << data.dump(2);
}

void Command::Project::getList_() const
{
    Redmine::API client{redmine_->getUrl(), redmine_->getToken()};
    
    auto projects = client.getProjects(limit_);

    printProjectList_(projects);

    return;
}
