/* 
* Copyright (C) 2024 Dominik Meyer
*  
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*  
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*  
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include "Redmine/Issue.hpp"
#include "Redmine/IssueStatus.hpp"
#include "Redmine/Project.hpp"
#include "nlohmann/json_fwd.hpp"
#include <Redmine/API.hpp>
#include <Redmine/Exception/Api.hpp>
#include <cstdint>
#include <exception>
#include <filesystem>
#include <iterator>
#include <sstream>
#include <stdexcept>
#include <vector>
#define LOGURU_WITH_STREAMS 1
#include <loguru.hpp>
#include <httplib.h>

Redmine::API::API(const std::string &serverURL, const std::string &authToken)
{

    if (serverURL.empty() || serverURL.size() < 7)
    {
        DLOG_S(ERROR) << "server URL not valid";

        throw std::invalid_argument("server url invalid");
    }

    if (!serverURL.starts_with("https://"))
    {
        DLOG_S(ERROR) << "server URL not starting with https://";

        throw std::invalid_argument("server URL does not start with https://");
    }

    redmineApiURL_=serverURL;

    if (authToken.empty() || authToken.length() < 10)
    {
        DLOG_S(ERROR) << "auth token not valid";

        throw std::invalid_argument("auth token is not valid");
    }   


    authToken_=authToken;
}

void Redmine::API::processGenericErrors_(httplib::Result &res) const
{
    if (res == nullptr)
    {
        LOG_S(FATAL) << "failed to get data from API endpoint";
        throw std::runtime_error("API request failed");
    }

    if (res->status == httplib::StatusCode::Unauthorized_401)
    {
        LOG_S(ERROR) << "authentication with API server failed (401)";
        throw std::runtime_error("authentication failed");
    }
    else if (res->status == httplib::StatusCode::NotFound_404)
    {
        LOG_S(ERROR) << "unknown API endpoint (404)";
        throw std::runtime_error("unknown API endpoint");
    } 
    else if (res->status == httplib::StatusCode::UnprocessableContent_422)
    {
        nlohmann::json error=nlohmann::json::parse(res->body);
        std::vector<std::string> errors=error["errors"].get<std::vector<std::string>>();
        std::stringstream strStream;
        for (auto e : errors) 
        {
            strStream << e << ",";
        }
        LOG_S(ERROR) << "Redmine API error unprocessible Content: " << strStream.str();
        throw Redmine::Exception::Api("Unprocessible Content");
    }
    else if (res->status != httplib::StatusCode::OK_200 && res->status != httplib::StatusCode::NoContent_204 && res->status != httplib::StatusCode::Created_201) 
    {
        LOG_S(ERROR) << "unsupported error " << httplib::status_message(res->status);
        throw std::runtime_error("unsupported error");
    } 

}

 nlohmann::json Redmine::API::get(const std::string  &path) const
 {
    DLOG_S(INFO) << "getting API endpoint " << path << " from " <<redmineApiURL_;
    httplib::Client client{redmineApiURL_};

    httplib::Headers headers = 
    {
        { "X-Redmine-API-Key", authToken_ },
        { "Content-Type", "application/json"}
    };

    auto res = client.Get(path , headers);

    processGenericErrors_(res);

    DLOG_S(INFO) << "get successful";
    return nlohmann::json::parse(res->body);
 }


void Redmine::API::put(const std::string  &path, const nlohmann::json &data) const
 {
    DLOG_S(INFO) << "putting API endpoint " << path << " from " <<redmineApiURL_;
    httplib::Client client{redmineApiURL_};
    client.set_basic_auth(authToken_, "");

    auto res = client.Put(path, data.dump(),"application/json");

    processGenericErrors_(res);
    
    DLOG_S(INFO) << "put successful";
 }

 void Redmine::API::post(const std::string  &path, const nlohmann::json &data) const
 {
    DLOG_S(INFO) << "posting to API endpoint " << path << " at " <<redmineApiURL_;
    httplib::Client client{redmineApiURL_};
    client.set_basic_auth(authToken_, "");

    auto res = client.Post(path, data.dump(),"application/json");

    processGenericErrors_(res);
    
    DLOG_S(INFO) << "post successful";
 }

std::string Redmine::API::upload(const std::filesystem::path &file, const std::string &filename) const
{
    httplib::Client client{redmineApiURL_};
    client.set_basic_auth(authToken_, "");

    if (!std::filesystem::exists(file))
    {
         LOG_S(ERROR) << "file " << file << " does not exist";
         throw std::runtime_error("file does not exist");
    }

    std::ifstream fileStream{file};
    std::stringstream fileBuffer;
    fileBuffer << fileStream.rdbuf();

    std::string path = "/uploads.json?filename=" + filename;
    DLOG_S(INFO) << "uploading file " << file.filename() << " to API endpoint " << path << " at " <<redmineApiURL_;

    auto res = client.Post(path, fileBuffer.str(),"application/octet-stream");

    processGenericErrors_(res);

    nlohmann::json response=nlohmann::json::parse(res->body);
    
    DLOG_S(INFO) << "upload successful";
    return response["upload"]["token"];
}


Redmine::User Redmine::API::getMyAccount() const 
{
    nlohmann::json userInfo = get("/my/account.json");

    return Redmine::User{userInfo};
}

void Redmine::API::setMyAccount(const Redmine::User &user) const
{

    put("/my/account.json", user.get());

}

std::vector<Redmine::Project> Redmine::API::getProjects(const std::uint32_t limit) const
{
    nlohmann::json projectList = get("/projects.json?limit="+std::to_string(limit));
    std::vector<Redmine::Project> projects;

    for (auto it = projectList["projects"].begin(); it != projectList["projects"].end(); ++it)
    {
        Redmine::Project p{*it};
        projects.push_back(p);
    }

    return projects; 
}


std::vector<Redmine::IssueStatus> Redmine::API::getIssueStatusList() 
{
    nlohmann::json statusList = get("/issue_statuses.json");
    issueStatusList_.clear();


    for (auto it=statusList["issue_statuses"].begin(); it != statusList["issue_statuses"].end(); ++it)
    {

        Redmine::IssueStatus status{*it};

        issueStatusList_.push_back(status);
    }

    return issueStatusList_;
}

bool Redmine::API::ready() const 
{   
    try {
        getMyAccount();
    } catch (const std::exception &e)
    {
        return false;
    }
    return true;
}

void Redmine::API::uploadFileToProject(const std::uint64_t projectId, const std::string &filePath, const std::string &fileName, const std::string &description, const std::uint32_t version) const
{

    std::string token = upload(filePath, fileName);

    nlohmann::json file;
    file["token"] = token;
    file["filename"] = fileName;
    file["description"] = description;
    file["version_id"]= version;
    nlohmann::json upload;
    upload["file"] = file;

    post("/projects/"+std::to_string(projectId)+"/files.json", upload);
}

 std::vector<Redmine::Issue> Redmine::API::getIssues(const Redmine::Filter& filter) const
 {
    std::vector<Redmine::Issue> issues;

    nlohmann::json issueList = get("/issues.json?" + filter.toQueryString());
    for (auto &issue : issueList["issues"])
    {
        issues.push_back(Redmine::Issue(issue));
    }

    return issues;
 }